/*
 * PanmirrorConstants.java
 *
 * Copyright (C) 2022 by Posit Software, PBC
 *
 * Unless you have received this program directly from Posit Software pursuant
 * to the terms of a commercial license agreement with Posit Software, then
 * this program is licensed to you under the terms of version 3 of the
 * GNU Affero General Public License. This program is distributed WITHOUT
 * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
 * AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
 *
 */
package org.rstudio.studio.client.panmirror;

public interface PanmirrorConstants extends com.google.gwt.i18n.client.Messages {
    String visualEditorText();
    String visualEditorLabel(String menuText);
    String formatTitle();
    String insertTitle();
    String tableTitle();
    String findReplaceTitle();
    String formatText();
    String insertText();
    String tableText();
    String panmirrorBlockMenuDefaultText();
    String panMirrorBlockMenuTitle();
    String undoMenuText();
    String redoMenuText();
    String selectAllMenuText();
    String boldMenuText();
    String italicMenuText();
    String codeMenuText();
    String strikeoutMenuText();
    String superscriptMenuText();
    String subscriptMenuText();
    String smallCapsMenuText();
    String underlineMenuText();
    String spanMenuText();
    String normalMenuText();
    String heading1MenuText();
    String heading2MenuText();
    String heading3MenuText();
    String heading4MenuText();
    String heading5MenuText();
    String heading6MenuText();
    String codeBlockMenuText();
    String codeBlockFormatMenuText();
    String blockquoteMenuText();
    String linkBlockMenuText();
    String divMenuText();
    String editAttributesMenuText();
    String clearFormattingMenuText();
    String texInlineMenuText();
    String texBlockMenuText();
    String htmlInlineMenuText();
    String htmlBlockMenuText();
    String rawInlineMenuText();
    String rawBlockMenuText();
    String expandChunkMenuText();
    String collapseChunkMenuText();
    String expandAllChunksMenuText();
    String collapseAllChunksMenuText();
    String bulletedListMenuText();
    String numberedListMenuText();
    String tightListMenuText();
    String sinkItemMenuText();
    String liftItemMenuText();
    String itemCheckboxMenuText();
    String itemCheckedMenuText();
    String listAttributesMenuText();
    String insertTableMenuText();
    String tableHeaderMenuText();
    String tableCaptionMenuText();
    String tableInsertColumnRightMenuText();
    String tableInsertColumnRightPluralMenuText();
    String tableInsertColumnLeftMenuText();
    String tableInsertColumnLeftPluralMenuText();
    String tableDeleteColumnMenuText();
    String tableDeleteColumnPluralMenuText();
    String tableInsertRowBelowMenuText();
    String tableInsertRowBelowPluralMenuText();
    String tableInsertRowAboveMenuText();
    String tableInsertRowAbovePluralMenuText();
    String tableDeleteRowMenuText();
    String tableDeleteRowPluralMenuText();
    String deleteTableMenuText();
    String tableNextCellMenuText();
    String tablePreviousCellMenuText();
    String tableAlignColumnLeftMenuText();
    String tableAlignColumnRightMenuText();
    String tableAlignColumnCenterMenuText();
    String tableAlignColumnDefaultMenuText();
    String anyMenuText();
    String tableMenuText();
    String linkMenuText();
    String removeLinkMenuText();
    String figureImageMenuText();
    String footnoteMenuText();
    String horizontalRuleMenuText();
    String paragraphMenuText();
    String commentMenuText();
    String yamlBlockMenuText();
    String shortcodeMenuText();
    String inlineMathMenuText();
    String displayMathMenuText();
    String definitionListMenuText();
    String termMenuText();
    String descriptionMenuText();
    String citationMenuText();
    String crossReferenceMenuText();
    String insertEmojiMenuText();
    String insertUnicodeMenuText();
    String insertEmDashMenuText();
    String insertEnDashMenuText();
    String insertNonBreakingSpaceMenuText();
    String insertHardLinkBreakMenuText();
    String insertTabsetMenuText();
    String insertCalloutMenuText();
    String goToNextSectionMenuText();
    String goToPreviousSectionMenuText();
    String goToNextChunkMenuText();
    String insertSlidePauseMenuText();
    String insertSlideNotesMenuText();
    String insertSlideColumnsMenuText();
    String editAttributesCaption();
    String spanAttributesCaption();
    String unwrapSpanRemoveButtonCaption();
    String divAttributesCaption();
    String calloutCaption();
    String unwrapDivTitle();
    String typeLabel();
    String appearanceLabel();
    String showIconLabel();
    String captionLabel();
    String optionalPlaceholder();
    String divTabList();
    String calloutText();
    String attributesText();
    String codeBlockText();
    String languageFormLabel();
    String optionalFormLabel();
    String figureLabel();
    String imageLabel();
    String widthLabel();
    String heightLabel();
    String autoText();
    String lockRatioText();
    String defaultAlignLabel();
    String leftLabel();
    String centerLabel();
    String rightLabel();
    String legendText();
    String alternativeTextLabel();
    String linkToLabel();
    String latexEnvironmentLabel();
    String titleAttributeLabel();
    String advancedLabel();
    String errorCaption();
    String errorMessage();
    String unitsLabel();
    String linkLabel();
    String removeLinkTitle();
    String textFormLabel();
    String titleToolTipLabel();
    String validateErrorMessage();
    String listLabel();
    String defaultChoiceList();
    String incrementalChoiceList();
    String nonIncrementalChoiceList();
    String exampleChoice();
    String decimalChoice();
    String editEquationCaption();
    String invalidIDCaption();
    String invalidIDMessage();
    String modelDialogCaption(String caption);
    String inlineText();
    String blockText();
    String removeFormatText();
    String validateCaption();
    String validateMessage();
    String imageChooserLabel();
    String browseLabel();
    String chooseImageCaption();
    String onProgressMessage();
    String errorValidateMessage();
    String citationErrorMessage();
    String uniqueCitationErrorMessage();
    String bibliographyErrorMessage();
    String bibliographyFileNameErrorMessage();
    String doiUnavailableCaption();
    String citationDOITitle();
    String citationFromText();
    String kUnknownError();
    String kNoDataError();
    String kServerError();
    String insertTableCaption();
    String insertTabsetCaption();
    String tabNamesFormLabel();
    String imageTabListLabel();
    String tabsText();
    String tabSetErrorMessage();
    String addTabCaptionInput(int index, String required);
    String optionalText();
    String formatLabel();
    String chooseFormatLabel();
    String rStudioGinjectorErrorMessage(int replaced);
    String chunkText(int sequence);
    String readingBibliographyProgressText();
    String savingBibliographyProgressText();
    String lookingUpDOIProgress();
    String loadingCollectionsProgressText();
    String readingCollectionsProgressText();
}
